# -*- coding: utf-8 -*-
from plone import api
from plone.dexterity.interfaces import IDexterityContainer
from plone.restapi.services import Service
from redturtle.bandi.vocabularies import TipologiaBandoVocabularyFactory
from zope.interface import implementer
from zope.publisher.interfaces import IPublishTraverse


@implementer(IPublishTraverse)
class BandiSearchFiltersGet(Service):
    def reply(self):
        """
        Return possible values based also on current user permissions
        """
        pc = api.portal.get_tool(name="portal_catalog")
        voc_tipologie = TipologiaBandoVocabularyFactory(self.context)

        tipologie = []
        subjects = []
        offices = []

        bandi_folder = None

        if IDexterityContainer.providedBy(self.context):
            bandi_folder = self.context

        if bandi_folder:
            bandi_folder_path = "/".join(bandi_folder.getPhysicalPath())
            query = {"portal_type": ["Bando"], "path": bandi_folder_path}
            brains = pc(query)

            for brain in brains:
                bando = brain.getObject()
                found = [x for x in tipologie if x["UID"] == bando.tipologia_bando]
                if not found:
                    tipologie.append(
                        {
                            "UID": bando.tipologia_bando,
                            "title": voc_tipologie.getTerm(bando.tipologia_bando).title,
                        }
                    )
                for sub in bando.subject:
                    found = [x for x in subjects if x["UID"] == sub]
                    if not found:
                        subjects.append({"UID": sub, "title": sub})

                for office_relation in bando.ufficio_responsabile:
                    offices.append(
                        {
                            "UID": office_relation.to_object.UID(),
                            "title": office_relation.to_object.title,
                        }
                    )
        else:
            for subject in pc.uniqueValuesFor("Subject_bando"):
                res = api.content.find(Subject_bando=subject)
                if res:
                    subjects.append({"UID": subject, "title": subject})

            for item in voc_tipologie.by_token:
                tipologie.append(
                    {"UID": item, "title": voc_tipologie.getTerm(item).title}
                )

            office_uids = pc.uniqueValuesFor("ufficio_responsabile_bando")
            offices = [{"UID": x.UID, "title": x.Title} for x in pc(UID=office_uids)]

        subjects.sort(key=lambda x: x["title"])
        offices.sort(key=lambda x: x["title"])
        tipologie.sort(key=lambda x: x["title"])
        return {
            "subjects": subjects,
            "offices": offices,
            "tipologie": tipologie,
        }



from plone.namedfile.browser import DisplayFile as BaseView
from urllib.parse import quote


class DisplayFile(BaseView):
    """
    Custom view
    """

    def set_headers(self, file):
        """
        backport of https://github.com/RedTurtle/redturtle.volto/pull/113

        We need to add filename to the reponse because otherwise the browser
        use the field name as filename (last path element).

        content-disposition should be "inline" to allow to display the file in the browser
        without forcing download (with "attachment" the browser will download it).
        """
        super().set_headers(file=file)

        filename = getattr(file, "filename", "")
        if filename is not None:
            if not isinstance(filename, str):
                filename = str(filename, "utf-8", errors="ignore")
            filename = quote(filename.encode("utf8"))
            self.request.response.setHeader(
                "Content-Disposition", f"inline; filename*=UTF-8''{filename}"
            )



# -*- coding: utf-8 -*-
from design.plone.policy.testing import (
    DESIGN_PLONE_POLICY_API_FUNCTIONAL_TESTING,
)
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.restapi.testing import RelativeSession
from zope.component import getUtility
from transaction import commit
from z3c.relationfield import RelationValue
from zope.intid.interfaces import IIntIds
from plone import api
from redturtle.bandi.vocabularies import TipologiaBandoVocabularyFactory

import unittest


class BandiSearchFiltersAPITest(unittest.TestCase):
    layer = DESIGN_PLONE_POLICY_API_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.anon_api_session = RelativeSession(self.portal_url)
        self.anon_api_session.headers.update({"Accept": "application/json"})

        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        voc_tipologie = TipologiaBandoVocabularyFactory(self.portal)
        keys = voc_tipologie.by_value.keys()
        tipologia_bando = tuple(keys)[0]

        intids = getUtility(IIntIds)

        self.bando_dir = api.content.create(
            container=self.portal, type="Folder", title="Bandi test folder"
        )

        self.uo_public_1 = api.content.create(
            container=self.portal, type="UnitaOrganizzativa", title="UO 1"
        )
        self.uo_public_2 = api.content.create(
            container=self.portal, type="UnitaOrganizzativa", title="UO 2"
        )
        self.uo_private = api.content.create(
            container=self.portal, type="UnitaOrganizzativa", title="UO 3"
        )
        self.bando_public_1 = api.content.create(
            container=self.portal,
            type="Bando",
            title="Bando 1",
            subject=("foo"),
            ufficio_responsabile=[RelationValue(intids.getId(self.uo_public_1))],
        )
        self.bando_public_2 = api.content.create(
            container=self.portal,
            type="Bando",
            title="Bando 2",
            subject=("foo", "bar"),
            ufficio_responsabile=[
                RelationValue(intids.getId(self.uo_public_2)),
                RelationValue(intids.getId(self.uo_private)),
            ],
        )
        self.bando_private = api.content.create(
            container=self.portal,
            type="Bando",
            title="Bando 3",
            subject=("foo", "baz"),
            ufficio_responsabile=[RelationValue(intids.getId(self.uo_public_1))],
        )

        self.bando_in_folder = api.content.create(
            container=self.bando_dir,
            type="Bando",
            title="Bando in folder",
            tipologia_bando=tipologia_bando,
            ufficio_responsabile=[RelationValue(intids.getId(self.uo_public_1))],
        )

        api.content.transition(obj=self.uo_public_1, transition="publish")
        api.content.transition(obj=self.uo_public_2, transition="publish")
        api.content.transition(obj=self.bando_public_1, transition="publish")
        api.content.transition(obj=self.bando_public_2, transition="publish")
        api.content.transition(obj=self.bando_in_folder, transition="publish")

        commit()

    def tearDown(self):
        self.api_session.close()
        self.anon_api_session.close()

    def test_endpoint_exists(self):
        response = self.api_session.get("/@bandi-search-filters")

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.headers.get("Content-Type"), "application/json")

    def test_endpoint_return_list_of_offices_based_on_permissions(self):
        response = self.api_session.get("/@bandi-search-filters").json()

        self.assertIn("offices", response)
        offices = response["offices"]

        self.assertEqual(len(offices), 3)

        response = self.anon_api_session.get("/@bandi-search-filters").json()

        self.assertIn("offices", response)
        offices = response["offices"]
        self.assertEqual(len(offices), 2)

    def test_endpoint_return_list_of_subjects_based_on_permissions(self):
        response = self.api_session.get("/@bandi-search-filters").json()

        self.assertIn("subjects", response)
        subjects = [x["UID"] for x in response["subjects"]]
        self.assertEqual(len(subjects), 3)
        self.assertEqual(subjects, ["bar", "baz", "foo"])

        response = self.anon_api_session.get("/@bandi-search-filters").json()

        self.assertIn("subjects", response)
        subjects = [x["UID"] for x in response["subjects"]]
        self.assertEqual(len(subjects), 2)
        self.assertEqual(subjects, ["bar", "foo"])

    def test_endpoint_return_related_office_for_bando_by_path(self):
        path = "/".join(self.bando_dir.getPhysicalPath()).replace("/plone/", "/")
        string_request = path + "/@bandi-search-filters"
        response = self.api_session.get(string_request).json()

        self.assertIn("offices", response)
        offices = [x["UID"] for x in response["offices"]]

        self.assertIn(self.uo_public_1.UID(), offices)
